(***************************************************************************
  HelpTest
  A demonstration program showing off the patched HelpFile
  PJB September 2, 1993, Internet mail to d91-pbr@nada.kth.se
  Copyright PJB 1993, All Rights Reserved. Portions Copyright Borland.
  Free source, use at your own risk.
  If modified, please state so if you pass this around.

  PreviousTopic is a magic help topic defined in HELPFILE.PAS

  This program is NOT meant to be a style guide, it's just a demonstration

***************************************************************************)
program HelpTest;
{$X+}

  uses
    App, Dialogs, Drivers, Menus, MsgBox, Objects, Views,
    Dos,
    HelpFile,
    HelpCtx;

  type
    TMain =
      object (TApplication)
        HelpInUse : boolean;

        function  ExeDir:PathStr;
        procedure GetEvent(var Event:TEvent); virtual;
        function  GetPalette:PPalette; virtual;
        procedure InitMenubar; virtual;
        procedure InitStatusLine; virtual;
        procedure HandleEvent(var Event:TEvent); virtual;
        procedure PopHelp(aHelpCtx:word);
      end;


  (*******************************************************************
    Use whatever values you like
  *******************************************************************)
  const
    ExeFileName  = 'HELPTEST.EXE';
    HelpFileName = 'HELPTEST.HLP';

    hcHelpWindow = 10000;

    cmHelp            = 9100;
    cmPreviousTopic   = 9101;
    cmHelpContents    = 9102;

    (* These two are one and the same in the BP IDE *)
    cmUsingHelp       = 9103;
    cmHelpOnHelp      = 9104;


  (*******************************************************************
    Return the directory of the main executable file
  *******************************************************************)
  function TMain.ExeDir;
    var
      EXEName : PathStr;
      Dir     : DirStr;
      Name    : NameStr;
      Ext     : ExtStr;
  begin
    if Lo(DosVersion)>=3 then
      EXEName:=ParamStr(0)
    else
      EXEName:=FSearch(ExeFileName, GetEnv('PATH'));
    FSplit(EXEName, Dir, Name, Ext);
    ExeDir:=Dir;
  end;


  (*******************************************************************
    Help popping and Status line support
  *******************************************************************)
  procedure TMain.GetEvent;
    procedure SwitchToTopic(Topic:word);
      var
        Event : TEvent;
    begin
      (* HelpInUse moved into the Application object *)
      if HelpInUse then
      begin
        Event.What:=evCommand;
        Event.Command:=cmSwitchToTopic;
        Event.InfoWord:=Topic;
        PutEvent(Event);
      end
      else
        PopHelp(Topic);
    end;
  begin
    inherited GetEvent(Event);

    if Event.What=evCommand then
    begin
      case Event.Command of
        (* The usual TV help command *)
        cmHelp:          PopHelp(GetHelpCtx);
        (* These are StatusLine commands and must reside here,
           else won't work inside (modal) Help *)
        cmPreviousTopic: SwitchToTopic(PreviousTopic);
        cmHelpContents:  SwitchToTopic(hcContents);
        cmHelpOnHelp:    SwitchToTopic(hcHelpOnHelp);
        else
          Exit;
      end;
      ClearEvent(Event);
    end;
  end;


  (*******************************************************************
    Standard Help palette
  *******************************************************************)
  function TMain.GetPalette;
    const
      CNewColor = CAppColor + CHelpColor;
      CNewBlackWhite = CAppBlackWhite + CHelpBlackWhite;
      CNewMonochrome = CAppMonochrome + CHelpMonochrome;
      P : array [apColor..apMonochrome] of string [Length(CNewColor)] =
        (CNewColor, CNewBlackWhite, CNewMonochrome);
  begin
    GetPalette := PPalette(@P[AppPalette]);
  end;


  (*******************************************************************
    Translate Help Menu commands
  *******************************************************************)
  procedure TMain.HandleEvent;
  begin
    inherited HandleEvent(Event);
    if Event.What=evCommand then
    begin
      case Event.Command of
        (* This command not handled by TMain.GetEvent *)
        cmUsingHelp:     PopHelp(hcUsingHelp); (* TP IDE uses hcHelpOnHelp *)
        else
          Exit;
      end;
      ClearEvent(Event);
    end;
  end;


  (*******************************************************************
    Demonstration Menu bar
  *******************************************************************)
  procedure TMain.InitMenubar;
    var
      R : TRect;
  begin
    GetExtent(R);
    R.B.Y:=R.A.Y+1;
    MenuBar:=New(PMenuBar, Init(R, NewMenu(
      NewSubMenu('~H~elp', hcNoContext, NewMenu(
        NewItem('~C~ontents', '', kbNoKey, cmHelpContents, hcNoContext,
        NewItem('~P~revious topic', '', kbNoKey, cmPreviousTopic, hcNoContext,
        NewItem('~U~sing help', '', kbNoKey, cmUsingHelp, hcNoContext,
      nil)))),
    nil))));
  end;


  (*******************************************************************
    Demonstration Status line. Use some DOS 6 Help keys
  *******************************************************************)
  procedure TMain.InitStatusLine;
    var
      R: TRect;
  begin
    GetExtent(R);
    R.A.Y := R.B.Y - 1;
    New(StatusLine, Init(R,
      NewStatusDef(hcHelpWindow, hcHelpWindow,
        StdStatusKeys(
        NewStatusKey('~F1~ Help on Help', kbF1, cmHelpOnHelp,
        NewStatusKey('~Alt+B~ Back', kbAltB, cmPreviousTopic,
        NewStatusKey('~Alt+C~ Contents', kbAltC, cmHelpContents,
        NewStatusKey('~F5~ Zoom', kbF5, cmZoom,
        NewStatusKey('~Esc~ Close help', kbEsc, cmClose,
      nil)))))),
      NewStatusDef(0, $FFFF,
        StdStatusKeys(
        NewStatusKey('~F1~ Help', kbF1, cmHelp,
        NewStatusKey('~Alt+X~ Exit', kbAltX, cmQuit,
      nil))),
      nil))));
  end;


  (*******************************************************************
    Pop up a (modal) help window, standard TVDEMO style
  *******************************************************************)
  procedure TMain.PopHelp;
    var
      W : PWindow;
      HFile : PHelpFile;
      HelpStrm : PDosStream;
  begin
    if not HelpInUse then
    begin
      HelpInUse:=True;

      New(HelpStrm, Init(FSearch(HelpFileName, ExeDir), stOpenRead));

      New(HFile, Init(HelpStrm));
      if HelpStrm^.Status<>stOk then
      begin
        MessageBox(^M^C'Could not open help file', nil, mfError+mfOkButton);
        Dispose(HFile, Done);
      end
      else
      begin
        W:=New(PHelpWindow, Init(HFile, aHelpCtx));
        if ValidView(W)<>nil then
        begin
          W^.HelpCtx:=hcHelpWindow;
          Application^.ExecView(W);
          Dispose(W, Done);
        end;
      end;
      HelpInUse:=False;
    end;
  end;


    (*******************************************************************
    *******************************************************************)

  var
    Main : TMain;

begin
  RegisterHelpFile;

  Main.Init;
  Main.Run;
  Main.Done;
end.
