(***************************************************************************
  NewMouse
  Snap-in replacement mouse cursor intended for Turbo Vision.
  PJB September 1, 1993, Internet mail to d91-pbr@nada.kth.se
  Copyright 1993, All Rights Reserved
  Free source, use at your own risk.
  If modified, please state so if you pass this around.

  Put this unit FIRST in your main program's unit list, and strange
  TEXT video modes will be recognized.
  This unit does NOT replace a resident mouse driver.
  Use UseNewMouse to toggle operation on/off, default is on.

  Don't switch video mode without hiding the mouse. Preferrably use
  Turbo Vision code to switch modes.
  To use graphics modes, first toggle NewMouse off with
  UseNewMouse(False). Don't forget to turn it back on.

  No hardware cursor support. You cannot change the appearance
  of the mouse cursor without turning NewMouse off.

  You cannot overlay this unit.

  This code does not seem to work well in 40-column modes with
  Microsoft mouse drivers, but they don't work either in 40 columns.

***************************************************************************)
unit NewMouse;

{$O-,R-,S-,X+}
{$C FIXED PRELOAD PERMANENT}

interface

  uses
    Dos,
   {$IFDEF DPMI}
    WinAPI,
   {$ENDIF}
    Drivers;

  procedure UseNewMouse(Yes:Boolean);


(***************************************************************************
***************************************************************************)
implementation


  (*******************************************************************
    Data segment variables
  *******************************************************************)
  const
    CrtWidth = $4A;

    OldPos   : Word = 1;
    Visible  : Byte = $FF;

  var
    Attr     : Byte;
    LastX    : Byte;
    LastY    : Byte;

    InstalledMouseProc : Pointer;


  (*******************************************************************
    Code segment variables to simplify assembler
  *******************************************************************)
  procedure OldInt33; assembler;
  asm
      db   0,0,0
  end;

  procedure NewMouseInUse; assembler; asm end;


  (*******************************************************************
    Called by original mouse driver with new mouse events
  *******************************************************************)
  procedure MouseProc; far; assembler;
  asm
      push ds

      mov  ax,SEG @Data
      mov  ds,ax

      push cx
      push dx
      call DWORD PTR InstalledMouseProc
      pop  dx
      pop  cx

      cmp  NewMouseInUse.Byte,0
      je   @Fin2

      mov  ax,SEG @Data
      mov  ds,ax

      mov  bx,cx
      mov  cl,3
      shr  bx,cl
      shr  dx,cl

      mov  LastX,bl
      mov  LastY,dl

      mov  es,Seg0040

      mov  al,es:[CrtWidth].Byte
      mul  dl
      add  bx,ax
      shl  bx,1
      inc  bx

      cli

      test Visible,80h
      je   @Show
      mov  OldPos,bx
      jmp  @Fin

    @Show:
      mov  es,Drivers.ScreenBuffer+2.Word

      mov  di,OldPos
      mov  al,Attr
      mov  es:[di],al

      mov  OldPos,bx

      mov  al,es:[bx]
      mov  Attr,al

      not  al
      and  al,77h
      mov  es:[bx],al

    @Fin:
      sti

    @Fin2:
      pop  ds
  end;


  (*******************************************************************
    Snooping mouse INT 33 replacement routines
  *******************************************************************)
  procedure HideMouse; assembler;
  asm
      sub  Visible,1
      jnc  @Fin

      mov  es,Drivers.ScreenBuffer+2.Word

      cli
      mov  bx,OldPos
      mov  al,Attr
      mov  es:[bx],al
      sti

    @Fin:
  end;

  procedure ShowMouse; assembler;
  asm
      cli
      inc  Visible
      jne  @Fin

      mov  es,Seg0040
      mov  al,es:[CrtWidth].Byte
      mul  LastY

      mov  bl,LastX
      mov  bh,0

      add  bx,ax
      shl  bx,1
      inc  bx
      mov  OldPos,bx

      mov  es,Drivers.ScreenBuffer+2.Word

      mov  al,es:[bx]
      mov  Attr,al
      not  al
      and  al,77h
      mov  es:[bx],al

    @Fin:
      sti
  end;

  procedure InstallProc; assembler;
  asm
      mov  ax,es
      or   ax,dx
      je   @Nil

      mov  InstalledMouseProc.Word, dx
      mov  InstalledMouseProc+2.Word, es

      push cs
      pop  es
      mov  dx,OFFSET MouseProc

    @Nil:
      mov  ax,12
      pushf
      call DWORD PTR [OldInt33]
  end;


  (*******************************************************************
    Mouse INT 33 snooping procedure
    Looks for HideMouse, ShowMouse and InstallEventHandler
  *******************************************************************)
  procedure NewInt33; far; assembler;
  asm
      cmp  NewMouseInUse.Byte,0
      je   @Skip

      cmp  ax,1
      je   @Local
      cmp  ax,2
      je   @Local
      cmp  ax,12
      je   @Local

    @Skip:
      jmp  DWORD PTR [OldInt33]

    @Local:
      push ax
      push bx
      push dx
      push ds
      push es

      mov  bx,SEG @Data
      mov  ds,bx

      cmp  ax,1
      jne  @L2
      call ShowMouse
      jmp  @Fin

    @L2:
      cmp  ax,2
      jne  @L3
      call HideMouse
      jmp  @Fin

    @L3:
      call InstallProc

    @Fin:
      pop  es
      pop  ds
      pop  dx
      pop  bx
      pop  ax
      iret
  end;


  (*******************************************************************
    DPMI Writable Code Selector
  *******************************************************************)
 {$IFDEF DPMI}
  var
    CodeSel : Word;
 {$ENDIF}


  (*******************************************************************
    Toggle NewMouse operation on/off
    Toggle off before you switch to graphics mode
  *******************************************************************)
  procedure UseNewMouse(Yes:Boolean);
  begin
    Drivers.HideMouse;

   {$IFDEF DPMI}
    Boolean(Mem[CodeSel:Ofs(NewMouseInUse)]):=Yes;
   {$ELSE}
    Boolean(@NewMouseInUse^):=Yes;
   {$ENDIF}

    Drivers.ShowMouse;
  end;


    (*******************************************************************
      Make sure we remove our snooping hook
    *******************************************************************)

  var
    SaveExit : Pointer;

  procedure ExitMouse; far;
  begin
    ExitProc:=SaveExit;

   {$IFDEF DPMI}
    SetIntVec($33, Ptr(CodeSel, Ofs(OldInt33)));
    FreeSelector(CodeSel);
   {$ELSE}
    SetIntVec($33, Pointer(@OldInt33^));
   {$ENDIF}
  end;


    (*******************************************************************
      Installation code
    *******************************************************************)

begin
 {$IFDEF DPMI}

  CodeSel:=AllocSelector(CSeg);
  PrestoChangoSelector(CSeg, CodeSel);

  Boolean(Mem[CodeSel:Ofs(NewMouseInUse)]):=True;
  GetIntVec($33, Pointer(MemL[CodeSel:Ofs(OldInt33)]));
  SetIntVec($33, @NewInt33);

 {$ELSE}

  Boolean(@NewMouseInUse^):=True;

  GetIntVec($33, Pointer(@OldInt33^));
  if Pointer(@OldInt33^)<>Nil then
    SetIntVec($33, @NewInt33);

 {$ENDIF}

  SaveExit:=ExitProc;
  ExitProc:=@ExitMouse;
end.