(***************************************************************************
  TVVideo
  Turbo Vision extended video modes support routines
  PJB August 30, 1993, Internet mail to d91-pbr@nada.kth.se
  Copyright PJB 1993, All Rights Reserved. Portions Copyright Borland.
  Free source, use at your own risk.
  If modified, please state so if you pass this around.

     PLEASE remember (I didn't) to use

      SetSpecialVideoMode      smSpecialFont8x8

      as in (for example)

          SetSpecialVideoMode(ScreenMode xor smSpecialFont8x8);

      not SetScreenMode, smFont8x8, if you want all to work as intended.

***************************************************************************)
unit TVVideo;
{$B-,O+,Q-,T-,X+}

interface

  uses
    App, Drivers, Objects, Memory, Views,
    Dos,
    Video;

  const
   {$IFDEF VesaSupport}
    smSpecialFont8x8 = $8000;        (* Don't use smFont8x8, please *)
   {$ELSE}
    smSpecialFont8x8 = smFont8x8;
   {$ENDIF}

  procedure PreventModeSwitch;
  procedure CalcScreenMode;
  procedure SetSpecialScreenMode(Mode:Word);
  procedure SetInternalFont(Font:byte);
  procedure InitTVVideo;


(***************************************************************************
***************************************************************************)
implementation


  (*******************************************************************
    From Borlands DRIVERS unit
  *******************************************************************)
  function GetCrtMode:Word; assembler;
  asm
      PUSH      BP

      MOV	AH,0FH
      INT       10H

      PUSH	AX
      MOV	AX,1130H
      MOV	BH,0
      MOV	DL,0
      INT       10H
      POP	AX

      MOV	DH,AH
      CMP	DL,25
      SBB	AH,AH
      INC	AH

      POP       BP
  end;

  (*******************************************************************
    Call this before InitVideo or DoneVideo to stop them from
    changing the video mode. This procedure *destroys* StartUpMode.
    Save StartUpMode (MyApp.Init; Save:=StartUpMode;) if you want to
    restore the video mode (StartUpMode:=Save; MyApp.Done;) on exit.

    Try this to keep the startup video mode (132-cols etc) active:
      begin
        if IsProbablyTextMode then PreventModeSwitch;
        MyApp.Init; MyApp.Run; MyApp.Done;
      end.
  *******************************************************************)
  procedure PreventModeSwitch;
  begin
    StartUpMode:=GetCrtMode;
    ScreenMode:=StartUpMode;
  end;


  (*******************************************************************
    Get true video mode, with Turbo Vision 8x8 flag
  *******************************************************************)
  procedure CalcScreenMode;
  begin
    ScreenMode:=GetSpecialVideoMode+
                smSpecialFont8x8*ord(Mem[Seg0040:CrtRows]>24);
  end;


  (*******************************************************************
    Center all views on the desktop
  *******************************************************************)
  procedure ReCenterDesktop;
    procedure ReCenter(P:PView); far;
      var
        X,Y : integer;
    begin
      X:=P^.Origin.X;
      Y:=P^.Origin.Y;

      if P^.Options and ofCenterX <> 0 then
        X:=(Desktop^.Size.X - P^.Size.X) div 2;
      if P^.Options and ofCenterY <> 0 then
        Y:=(Desktop^.Size.Y - P^.Size.Y) div 2;

      P^.MoveTo(X, Y);
    end;
  begin
    Desktop^.ForEach(@ReCenter);
  end;


  (*******************************************************************
    Use this procedure to change video mode instead of SetScreenMode
    which will not set modes other than 2,3 and 7.

    DON'T use SetScreenMode if you use SetSpecialVideoMode.
    The display wont be redrawn if the screen size doesn't change.
    This is a design flaw at the heart of Turbo Vision
  *******************************************************************)
  procedure SetSpecialScreenMode(Mode:Word);
    const
      CrtInfo = $87;
  begin
    HideMouse;

    SetSpecialVideoMode(Mode and not smSpecialFont8x8);

    if Mode and smSpecialFont8x8<>0 then
    asm                        (* From Drivers unit *)
        PUSH    BP

	MOV	AX,1112h       (* Use CGA 8x8 font *)
	MOV	BL,0
	INT     10h

	MOV	AX,1130h       (* Get number of screen rows less one *)
	MOV	BH,0
	MOV	DL,0
	INT     10h

	CMP	DL,42
	JNE	@Fin           (* Jump if not an EGA (bad test anyway) *)

        MOV     ES,Seg0040
	OR	ES:[CrtInfo].Byte,1  (* Disable CGA cursor emulation *)
	MOV	AH,1
	MOV	CX,600h        (* Set cursor size: Start 6, End 0 *)
	INT     10h

	MOV	AH,12H         (* Use alternate PrtScr handler *)
	MOV	BL,20H
	INT     10h

      @Fin:
        POP     BP
    end;

    InitTVVideo;
  end;


  (*******************************************************************
    TV wrapper for Video.UseInternalFont
  *******************************************************************)
  procedure SetInternalFont(Font:byte);
  begin
    HideMouse;
    UseInternalFont(Font);
    InitTVVideo;
  end;


  (*******************************************************************
    This is separate procedure so we can use it for font changes etc
  *******************************************************************)
  procedure InitTVVideo;
    var
      R : TRect;
  begin
    PreventModeSwitch;         (* Disable InitVideo mode switch *)
    InitVideo;                 (* Recalc CRT data *only* *)

    DoneMemory;                (* Dispose of cache buffers *)
    InitMemory;
    Application^.InitScreen;   (* Calculate shadow sizes (debatable) *)

    Application^.Buffer:=Nil;  (* Disable all screen writing *)

    R.Assign(0, 0, ScreenWidth, ScreenHeight);
    Application^.ChangeBounds(R);    (* Resize application *)
    ReCenterDesktop;                 (* Center desktop items *)

    if IsColorMode then     (* Let's hope this works *)
      PtrRec(ScreenBuffer).Seg:=SegB800
    else
    begin
      ShadowSize.X := 0;
      ShadowSize.Y := 0;
      ShowMarkers := True;
      AppPalette := apMonochrome;
      PtrRec(ScreenBuffer).Seg:=SegB000;
    end;
    Application^.Buffer:=ScreenBuffer;

    Application^.Redraw;        (* Draw menubar, desktop and statusline *)
    ShowMouse;
    CalcScreenMode;
  end;


    (*******************************************************************
    *******************************************************************)

end.
