(***************************************************************************
  VideoTest
  Demonstrates the Video, ModeDlg and TVVideo units
  PJB October 8, 1993, Internet mail to d91-pbr@nada.kth.se
  Copyright 1993, All Rights Reserved
  Free source, use at your own risk.
  If modified, please state so if you pass this around.

   To find anything interesting on a Video7 VGA, you have to define
    Video7Support and rebuild, unless you have VESA support of course.

   There is VESA support, but you must define VesaSupport to get it.

   Try defining Color to get a colorful Info Dialog. This might not
    work if you have modified TView.GetColor in the TV library.

  What this demonstration program does:
    The menu bar lets you choose between File, Video and Info.
    The Video menu lets you toggle between 25 and 43/50 lines, select
    a video mode and choose between internal video card fonts.

    If you choose Select video mode, you will see some warnings.
    Read them, press the "Run test" button and wait while the program
    scans for available video modes. Don't be alerted if your screen
    flashes. If you do not see a dialog box within four seconds, the
    computer probably crashed. This is a problem with very old (usually
    EGA) video BIOSes.

    If you have full VESA support you will instantly see the available
    video modes without any flashing. UNIVESA by Kendall Bennett does
    not provide text mode support, so there will be flashing if there is
    no additional VESA support.

    The dialog shows a list of text modes found. Press Enter to
    preview a video mode for one second (if your monitor goes bonkers,
    just wait for one second and the screen will be restored. This
    means your monitor cannot cope with that video mode's resolution
    (e.g. displaying 60 lines with a VGA card on an standard EGA monitor)

    The following has been known to happen with fixed-sync VGAs:
  (my fixed sync EGA monitor copes, and multi-sync VGAs have no problems)
    If your monitor emits "cracks" while in this video mode, your screen
    fuse (easily replaced) might melt if the monitor is subjected to
    that video mode for a longer period of time. You can always turn the
    monitor off if you think it is breaking down.

     Your mouse driver probably won't manage with extended video modes
    (especially not if it is a Microsoft driver). Check out the NewMouse
    unit supplied: Uncomment NewMouse in the uses statement below.

    This program has been tested on a variety of machines with different
    hardware and video cards without any problems.

***************************************************************************)
program VideoTst;
{$B-,Q-,T-,X+,V-}

  uses
    {NewMouse,}    (* Use NewMouse to keep your mouse pointer happy *)
    App, Dialogs, Drivers, Memory, Menus, MsgBox, Objects, Views,
    Dos,
   {$IFDEF Color}
    ColorTxt,
   {$ENDIF}
    ModeDlg, Video, TVVideo;

  type
    TMain =
      object (TApplication)
        DosVideoState : VideoState;
        TVVideoState : VideoState;

        constructor Init;
        destructor  Done; virtual;
        procedure DosShell;
        procedure InitMenubar; virtual;
        procedure HandleEvent(var Event:TEvent); virtual;
        function  MakeVideoInfoDialog : PDialog;
      end;


  (*******************************************************************
    Demo commands
  *******************************************************************)
  const
    cm14p        = 101;
    cm16p        = 102;
    cmVideoMode  = 103;
    cmVideoLines = 104;
    cmVideoInfo  = 105;


  (*******************************************************************
    Include warnings dialog
  *******************************************************************)
  {$I IMPRTANT.PAS}


  (*******************************************************************
    Init video system, save initial video mode
  *******************************************************************)
  constructor TMain.Init;
    var
      InitState : VideoState;
  begin
    (* Always start with this command *)
    CheckVideoType;

    InitState.Save;             (* Use temporary variable since... *)
    inherited Init;             (* ... this zeroes the whole object *)
    DosVideoState:=InitState;

    (* Disable some features on non VGA cards *)
    if VideoType=Other then
      DisableCommands([cmVideoMode, cmVideoLines, cm14p, cm16p]);
    if VideoType=EGA then
      DisableCommands([cm16p]);

    (* Set ScreenMode to a value closer to reality (for V7,VESA) *)
    CalcScreenMode;

    if VideoType=Other then
      MessageBox('This program intended for EGA/VGA', nil, mfInformation+mfOKButton);
  end;


  (*******************************************************************
    Restore initial video mode
  *******************************************************************)
  destructor TMain.Done;
  begin
    TVVideo.PreventModeSwitch;
    inherited Done;
    DosVideoState.Restore;
  end;


  (*******************************************************************
    New DosShell procedure
  *******************************************************************)
  procedure TMain.DosShell;
  begin
    DoneSysError;
    DoneEvents;

    TVVideoState.Save;
    DosVideoState.Restore;

    DoneDosMem;
    WriteShellMsg;
    SwapVectors;
    Exec(GetEnv('COMSPEC'), '');
    SwapVectors;
    InitDosMem;

    DosVideoState.Save;
    TVVideoState.Restore;
    InitTVVideo;

    InitEvents;
    InitSysError;
    Redraw;
  end;


  (*******************************************************************
    Video mode commands

    This code is all trouble...
    cmDosShell must be overridden, since DosShell is not a virtual
    procedure (yuck, Borland) so we must check for cmDosShell before
    we call TApplication.HandleEvent. Instead, I call TProgram's
    HandleEvent first and deal with cmDosShell afterwards.
    It gets very messy if you promptly want to call
    TApplication.HandleEvent. You have to put it last in this procedure
    and use two calls to inherited HandleEvent (or use goto :-) or a flag)
    Or you could check for cmDosShell first, call inherited HandleEvent
    and then do the big evCommand check.
  *******************************************************************)
  procedure TMain.HandleEvent;
  begin
    (* No cmDosShell, cmTile, cmCascade processing in TProgram *)
    TProgram.HandleEvent(Event);

    if Event.What=evCommand then
    begin
      case Event.Command of
        cm14p:  TVVideo.SetInternalFont(Internal8x14Font);
        cm16p:  TVVideo.SetInternalFont(Internal8x16Font);

        (* Copied from TApplication.HandleEvent *)
        cmTile:        Tile;
        cmCascade:     Cascade;
        cmDosShell:    DosShell;  (* MUST be overridden *)

        cmVideoMode:
          if not HasToScan or  (* Don't warn if modes already known *)
             (Application^.ExecuteDialog(MakeImportantDialog, nil)=cmOK) then
          begin
            SetUpVideoList;
            SelectVideoModeDialog;
          end;
        cmVideoLines:  SetSpecialScreenMode(ScreenMode xor smSpecialFont8x8);
        cmVideoInfo:   ExecuteDialog(MakeVideoInfoDialog, Nil);
        else
          Exit;
      end;
      ClearEvent(Event);
    end;
  end;

  (*******************************************************************
    Menu bar
  *******************************************************************)
  procedure TMain.InitMenubar;
    var
      R : TRect;
  begin
    GetExtent(R);
    R.B.Y:=R.A.Y+1;
    MenuBar:=New(PMenuBar, Init(R, NewMenu(
      NewSubMenu('~F~ile', hcNoContext, NewMenu(
        NewItem('~D~OS shell', '', kbNoKey, cmDosShell, hcDosShell,
        NewItem('E~x~it', 'Alt+X', kbAltX, cmQuit, hcExit,
        nil))),
      NewSubMenu('~V~ideo', hcNoContext, NewMenu(
        NewItem('Toggle video ~l~ines', '', kbNoKey, cmVideoLines, hcNoContext,
        NewItem('Select video ~m~ode...', '', kbNoKey, cmVideoMode, hcNoContext,
        NewLine(
        NewItem('Internal 1~4~p font', '', kbNoKey, cm14p, hcNoContext,
        NewItem('Internal 1~6~p font', '', kbNoKey, cm16p, hcNoContext,
        nil)))))),
      NewItem('~I~nfo', '', kbNoKey, cmVideoInfo, hcNoContext,
    nil))))));
  end;


  (*******************************************************************
    Create the video information dialog
  *******************************************************************)
  function TMain.MakeVideoInfoDialog : PDialog;
    var
      R       : TRect;
      Dlg     : PDialog;
      Control : PView;
      TempStr : string;
      DataArr : array [0..3] of LongInt;

    procedure AddInfo(const S:String; Color:Byte);
   {$IFDEF Color}
      var
        Text    : PColoredText;
    begin
      New(Text, Init(R, S, Color));
      Dlg^.Insert(Text);
      AddShadowTo(Text);
    end;
   {$ELSE}
      var
        Text    : PStaticText;
    begin
      New(Text, Init(R, S));
      Dlg^.Insert(Text);
    end;
   {$ENDIF}

  begin
    R.Assign(0, 0, 65, 14);
    New(Dlg, Init(R, 'Video Info'));
    Dlg^.Options := Dlg^.Options or ofCentered;

    R.Assign(4, 2, 61, 3);

   {$IFDEF Color}
    (* Check to see if TV library has been modified *)
    if Word(Ptr(Seg(TView.GetColor), Ofs(TView.GetColor))^)<>$8955 then
    begin
      MessageBox(^C'OOPS!'^M^C'You''ve changed TView.GetColor. There is now a bug in COLORTXT.PAS',
        Nil, mfError+mfOKButton);
      Dlg^.Free;
      MakeVideoInfoDialog := Nil;
      Exit;
    end;
   {$ENDIF}

    case VideoType of
      VGA:   AddInfo(^C'VGA detected', $1F);
      EGA:   AddInfo(^C'EGA detected', $1F);
      Other: AddInfo(^C'EGA/VGA not detected', $1F);
    end;


    R.Move(0, 2);
    DataArr[0]:=GetSpecialVideoMode;
    DataArr[1]:=Mem[Seg0040:CrtWidth];
    DataArr[2]:=Mem[Seg0040:CrtRows]+1;
    DataArr[3]:=Mem[Seg0040:CrtPoints];
    FormatStr(TempStr, '  Mode: %xh   Width: %d   Height: %d   CharHeight: %d', DataArr);
    AddInfo(TempStr, $1F);


    R.Move(0, 2);
   {$IFDEF VesaSupport}
    DataArr[0]:=Hi(VESA.VesaVersion);  (* Compilation Error? Use Build! *)
    DataArr[1]:=Lo(VESA.VesaVersion);
    FormatStr(TempStr, '  VESA version %d.%d detected', DataArr);

    if VESA.VesaVersion=0 then
      AddInfo('  VESA support not detected', $3F)
    else
      AddInfo(TempStr, $3F);
   {$ELSE}
    AddInfo('  VESA code not compiled', $4F);
   {$ENDIF}


    R.Move(0, 2);
   {$IFDEF Video7Support}
    if Video7 then           (* Compilation Error? Use Build! *)
      AddInfo('  Video 7 detected', $3F)
    else
      AddInfo('  Video 7 not detected', $3F);
   {$ELSE}
      AddInfo('  Video 7 code not compiled', $4F);
   {$ENDIF}


    R.Assign(27, 11, 37, 13);
    Control := New(PButton, Init(R, 'O~K~', cmOK, bfDefault));
    Dlg^.Insert(Control);

    Dlg^.SelectNext(False);
    MakeVideoInfoDialog := Dlg;
  end;

    (*******************************************************************
    *******************************************************************)

  var
    Main : TMain;

begin
  Main.Init;
  Main.Run;
  Main.Done;
end.
